package vrp;

import java.util.ArrayList;
import java.util.List;

import org.graphstream.graph.Edge;
import org.graphstream.graph.Node;
import org.graphstream.ui.spriteManager.Sprite;
import org.graphstream.ui.spriteManager.SpriteManager;

/**
 * Displays a solution in a viewer. This class plays the scenario provided by a
 * VRP solution. Vehicles are visualized as sprites moving in the graph.
 * 
 * @author Stefan Balev
 * 
 */
public class SolutionPlayer {
	protected class Vehicle {
		protected Sprite s;
		protected List<Node> path;
		protected int nodeIndex;
		protected int dir;
		protected double pos;
		protected Edge edge;

		protected Vehicle(int i) {
			path = new ArrayList<Node>();
			addPath(sol.getGraph().getWarehouse(), sol.getClient(i, 0));
			for (int j = 1; j < sol.getClientCount(i); j++)
				addPath(sol.getClient(i, j - 1), sol.getClient(i, j));
			addPath(sol.getClient(i, sol.getClientCount(i) - 1), sol.getGraph()
					.getWarehouse());
			path.add(sol.getGraph().getWarehouse());

			s = sm.addSprite(i + "");
			nodeIndex = 0;
			edge = path.get(0).getEdgeToward(path.get(1));
			s.attachToEdge(edge.getId());
			if (edge.getSourceNode() == path.get(0)) {
				dir = 1;
				pos = 0;
			} else {
				dir = -1;
				pos = 1;
			}
			s.setPosition(pos);
			s.addAttribute("ui.color", i / (sol.getVehicleCount() - 1.0));
		}

		protected void addPath(Node from, Node to) {
			for (Node node : sol.getGraph().getDijkstra(to).getPathNodes(from))
				path.add(node);
			path.remove(path.size() - 1);
		}

		protected boolean step() {
			if (edge == null)
				return false;
			if ((dir == 1 && pos == 1) || (dir == -1 && pos == 0)) {
				if (++nodeIndex == path.size() - 1) {
					edge = null;
					return false;
				}
				edge = path.get(nodeIndex).getEdgeToward(
						path.get(nodeIndex + 1));
				if (edge.getSourceNode() == path.get(nodeIndex)) {
					dir = 1;
					pos = 0;
				} else {
					dir = -1;
					pos = 1;
				}
			}
			pos += dir * speed / edge.getNumber("length");
			if (pos < 0)
				pos = 0;
			if (pos > 1)
				pos = 1;
			s.attachToEdge(edge.getId());
			s.setPosition(pos);
			return true;
		}
	}

	protected Solution sol;
	protected SpriteManager sm;
	protected double speed = 10;
	protected Vehicle[] vehicles;

	/**
	 * Creates a solution player
	 * 
	 * @param sol
	 *            the solution to be played
	 */
	public SolutionPlayer(Solution sol) {
		this.sol = sol;
		sm = new SpriteManager(sol.getGraph());
		vehicles = new Vehicle[sol.getVehicleCount()];
		for (int i = 0; i < sol.getVehicleCount(); i++)
			vehicles[i] = new Vehicle(i);

	}

	/**
	 * Sets the speed of the vehicles
	 * 
	 * @param speed
	 *            The speed expressed as length per iteration
	 */
	public void setSpeed(double speed) {
		this.speed = speed;
	}

	protected boolean step() {
		boolean more = false;
		for (int i = 0; i < sol.getVehicleCount(); i++)
			more = vehicles[i].step() || more;
		return more;
	}

	/**
	 * Animates vehicle sprites
	 */
	public void play() {
		while (step())
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
	}
}
